#!/usr/bin/env python
# -*- coding: CP949 -*-
#
# generated by wxGlade 0.7.2 on Sat Jul 30 12:35:25 2016
#


# Copyright: Kee Hwan Nam, KAIST, Daejeon, Republic of Korea
# email: snowall@kaist.ac.kr / snowall@gmail.com
# This program is licenced under GPL v3
# See https://www.gnu.org/licenses/gpl-3.0.html

# Any bug report or comment is welcomed. Please do not hasitate to contact me.

# This program requires wxPython, numpy, scipy libraries in Python 2.7

# This program calculates optical modes propagating in three-layer optical waveguide.

# version: 0.1
#~ import mpmath as np

#~ np.dps=30
import mpmath
mpmath.dps=30
import time
import multiprocessing
import wx
import os
import psutil
# begin wxGlade: dependencies
import gettext

# end wxGlade
# begin wxGlade: extracode
# end wxGlade
import numpy as np

#~ mpmath.seterr(all='raise')
np.seterr(all='ignore')

import numpy.linalg as linalg
import scipy.special as sp
import scipy.constants
from scipy.special import jv as jv
from scipy.special import yv as yv
from scipy.special import kv as kv
from scipy.special import iv as iv
from scipy.special import jvp as jvp
from scipy.special import yvp as yvp
from scipy.special import kvp as kvp
from scipy.special import ivp as ivp
from scipy.special import jve as jve
from scipy.special import yve as yve
from scipy.special import kve as kve
from scipy.special import ive as ive


#~ jv = lambda n,x:mpmath.autoprec(mpmath.besselj)(n,x)
#~ yv = lambda n,x:mpmath.autoprec(mpmath.bessely)(n,x)
#~ iv = lambda n,x:mpmath.autoprec(mpmath.besseli)(n,x)
#~ kv = lambda n,x:mpmath.autoprec(mpmath.besselk)(n,x)
#~ jvp = lambda n,x:mpmath.autoprec(mpmath.besselj)(n,x,derivative=1)
#~ yvp = lambda n,x:mpmath.autoprec(mpmath.bessely)(n,x,derivative=1)
#~ ivp = lambda n,x:mpmath.autoprec(mpmath.besseli)(n,x,derivative=1)
#~ kvp = lambda n,x:mpmath.autoprec(mpmath.besselk)(n,x,derivative=1)




from scipy.optimize import brentq as rootfind
#~ from mpmath import findroot as rootfind
mu=scipy.constants.mu_0
ep=scipy.constants.epsilon_0
speed=scipy.constants.speed_of_light

def modeIdentify(p,q):
	if p==0:
		if q%2==1:
			return "TM0"+str((q+1)/2)
		else:
			return "TE0"+str(q/2)
	else:
		if q%2==1:
			return "HE"+str(p)+str((q+1)/2)
		else:
			return "EH"+str(p)+str(q/2)
			
	return "No such mode!\n"

class mode(multiprocessing.Process):
	modes=[]
	def __init__(self,radialmodes,finished,materials, **kwds):
		multiprocessing.Process.__init__(self)
		self.finished=finished
		self.modes=[]
		self.radialmodes=radialmodes
		self.r1=float(kwds.pop('r1'))*scipy.constants.micro/2.
		self.r2=float(kwds.pop('r2'))*scipy.constants.micro/2.
		self.wl=float(kwds.pop('wavelength'))/1000.
		self.m=int(kwds.pop('mode_number'))
		self.step=1./float(kwds.pop('step'))
		self.n1=materials['core']
		self.n2=materials['clad']
		self.n3=materials['air']
		self.limit=int(kwds.pop('limit'))
		self.k = np.pi*2000000./self.wl # wavenumber
		self.o = speed*self.k # angular frequency of the light.
		self.oep=self.o*ep
		self.omu=self.o*mu
	def run(self):
		try:
			
			self.n1=eval("lambda y:"+(self.n1))
			self.n2=eval("lambda y:"+(self.n2))
			self.n3=eval("lambda y:"+(self.n3))

			self.n_1,self.n_2,self.n_3=self.n1(self.wl),self.n2(self.wl),self.n3(self.wl)
			self.n_1*=self.n_1
			self.n_2*=self.n_2
			self.n_3*=self.n_3
			indexStep=(self.n1(self.wl)-self.n2(self.wl))/self.step
			j=(self.n1(self.wl))-indexStep
			
			myVal0=self.layerdeterminant(j)
			myVal=myVal0
			v=1
			#~ targetfile=open("mp"+str(os.getpid())+".txt","a")
			for i in np.arange(self.n1(self.wl)-indexStep,self.n2(self.wl)+indexStep,-indexStep):
				myVal0=myVal
				myVal=self.layerdeterminant((i))
				#~ print type(myVal),myVal , i
				#~ targetfile.writelines(str(self.wl)+","+str(i)+","+str(myVal[1])+"\n")
				if myVal*myVal0<0.:	
					self.modes+=[[modeIdentify(self.m,v),rootfind(self.layerdeterminant,(i),(j))]]
					v+=1
				if v>self.limit:
					break
				j=i
			indexStep=(self.n2(self.wl)-self.n3(self.wl))/self.step
			j=(self.n3(self.wl))
			myVal0=self.layerdeterminant((i))
			myVal=myVal0
			for i in np.arange(self.n2(self.wl)-indexStep,self.n3(self.wl)+indexStep,-indexStep):
				myVal0=myVal
				myVal=self.layerdeterminant((i))
				if myVal*myVal0<0.:	
					self.modes+=[[modeIdentify(self.m,v),rootfind(self.layerdeterminant,(i),(j))]]
					v+=1
				if v>self.limit:
					break
				j=i
		finally:
			self.radialmodes.append([self.wl,self.modes])
			self.finished.append(1)
		
	def showparameters(self):
		print "r1",self.r1
		print "r2",self.r2
		print "wl",self.wl
		print "mode",self.m
		print "n1",self.n1(self.wl)
		print "n2",self.n2(self.wl)
		print "n3",self.n3(self.wl)
			
	def layerdeterminant(self,nef):
		# boundary condition
		self.beta=self.k*nef
		ne=nef*nef
		s1=self.k*np.sqrt(self.n_1-ne) # core
		if self.n_2>ne:
			s2=self.k*np.sqrt(self.n_2-ne) # clad mode
		else:
			s2=self.k*np.sqrt(ne-self.n_2) # core mode
		s3=self.k*np.sqrt(ne-self.n_3) # clad
		
		ss1=self.k*self.k*(self.n_1-ne)
		ss2=self.k*self.k*(self.n_2-ne)
		ss3=self.k*self.k*(self.n_3-ne)
		def e_r_a1(n,r,b):
			return s1*b*jvp(n,s1*r)/(ss1*jv(n,s1*self.r1))

		def e_r_c1(n,r,b):
			return self.o*mu*n*jv(n,s1*r)/(ss1*r*jv(n,s1*self.r1))
				
		# e_theta
		def e_t_a1(n,r,b):
			return -b*n*jv(n,s1*r)/(ss1*r*jv(n,s1*self.r1))

		def e_t_c1(n,r,b):
			return -s1*self.o*mu*jvp(n,s1*r)/(ss1*jv(n,s1*self.r1))

		# e_z
		def e_z_a1(n,r,b):
			return jv(n,s1*r)/jv(n,s1*self.r1)

		# h_r	
		def h_r_c1(n,r,b):
			return s1*b*jvp(n,s1*r)/(ss1*jv(n,s1*self.r1))
			
		def h_r_a1(n,r,b):
			return self.o*ep*n*self.n_1*jv(n,s1*r)/(ss1*r*jv(n,s1*self.r1))

		# h_theta
		def h_t_c1(n,r,b):
			return b*n*jv(n,s1*r)/(ss1*r*jv(n,s1*self.r1))
			
		def h_t_a1(n,r,b):
			return s1*self.o*ep*self.n_1*jvp(n,s1*r)/(ss1*jv(n,s1*self.r1))
			
		#h_z
		def h_z_c1(n,r,b):
			return jv(n,s1*r)/jv(n,s1*self.r1)

		# inner cladding core mode
		# if clad mode: iv, kv, self.r2 -> jv, yv, self.r1, repectively.

		def e_r_a2_core(n,r,b):
			return s2*b*ivp(n,s2*r)/(ss2*iv(n,s2*self.r2))

		def e_r_b2_core(n,r,b):
			return s2*b*kvp(n,s2*r)/(ss2*kv(n,s2*self.r2))

		def e_r_c2_core(n,r,b):
			return self.o*mu*n*iv(n,s2*r)/(ss2*r*iv(n,s2*self.r2))

		def e_r_d2_core(n,r,b):
			return self.o*mu*n*kv(n,s2*r)/(ss2*r*kv(n,s2*self.r2))


		def e_t_a2_core(n,r,b):
			return -b*n*iv(n,s2*r)/(ss2*r*iv(n,s2*self.r2))
			
		def e_t_b2_core(n,r,b):
			return -b*n*kv(n,s2*r)/(ss2*r*kv(n,s2*self.r2))

		def e_t_c2_core(n,r,b):
			return -s2*self.o*mu*ivp(n,s2*r)/(ss2*iv(n,s2*self.r2))
			
		def e_t_d2_core(n,r,b):
			return -s2*self.o*mu*kvp(n,s2*r)/(ss2*kv(n,s2*self.r2))
			

		def e_z_a2_core(n,r,b):
			#~ print type(s2),type(r),type(self.r2)
			return iv(n,s2*r)/iv(n,s2*self.r2)
			
		def e_z_b2_core(n,r,b):
			return kv(n,s2*r)/kv(n,s2*self.r2)


		def h_r_c2_core(n,r,b):
			return s2*b*iv(n,s2*r)/(ss2*iv(n,s2*self.r2))

		def h_r_d2_core(n,r,b):
			return s2*b*kv(n,s2*r)/(ss2*kv(n,s2*self.r2))
			
		def h_r_a2_core(n,r,b):
			return self.o*ep*self.n_2*n*iv(n,s2*r)/(ss2*r*iv(n,s*self.r2))
			
		def h_r_b2_core(n,r,b):
			return self.o*ep*self.n_2*n*kv(n,s2*r)/(ss2*r*kv(n,s*self.r2))
			

		def h_t_c2_core(n,r,b):
			return b*n*iv(n,s2*r)/(ss2*r*iv(n,s2*self.r2))
			
		def h_t_d2_core(n,r,b):
			return b*n*kv(n,s2*r)/(ss2*r*kv(n,s2*self.r2))
			
		def h_t_a2_core(n,r,b):
			return s2*self.o*ep*self.n_2*ivp(n,s2*r)/(ss2*iv(n,s2*self.r2))

		def h_t_b2_core(n,r,b):
			return s2*self.o*ep*self.n_2*kvp(n,s2*r)/(ss2*kv(n,s2*self.r2))
			

		def h_z_c2_core(n,r,b):
			#~ print iv(n,s2*r)/iv(n,s2*self.r2)
			return iv(n,s2*r)/iv(n,s2*self.r2)

		def h_z_d2_core(n,r,b):
			return kv(n,s2*r)/kv(n,s2*self.r2)

		# inner cladding core mode
		# if clad mode: iv, kv, self.r2 -> jv, yv, self.r1, repectively.

		def e_r_a2_clad(n,r,b):
			return s2*b*jvp(n,s2*r)/(ss2*jv(n,s2*self.r2))

		def e_r_b2_clad(n,r,b):
			return s2*b*yvp(n,s2*r)/(ss2*yv(n,s2*self.r2))

		def e_r_c2_clad(n,r,b):
			return self.o*mu*n*jv(n,s2*r)/(ss2*r*jv(n,s2*self.r2))

		def e_r_d2_clad(n,r,b):
			return self.o*mu*n*yv(n,s2*r)/(ss2*r*yv(n,s2*self.r2))


		def e_t_a2_clad(n,r,b):
			return -b*n*jv(n,s2*r)/(ss2*r*jv(n,s2*self.r2))
			
		def e_t_b2_clad(n,r,b):
			return -b*n*yv(n,s2*r)/(ss2*r*yv(n,s2*self.r2))

		def e_t_c2_clad(n,r,b):
			return -s2*self.o*mu*jvp(n,s2*r)/(ss2*jv(n,s2*self.r2))
			
		def e_t_d2_clad(n,r,b):
			return -s2*self.o*mu*yvp(n,s2*r)/(ss2*yv(n,s2*self.r2))
			

		def e_z_a2_clad(n,r,b):
			return jv(n,s2*r)/jv(n,s2*self.r2)
			
		def e_z_b2_clad(n,r,b):
			return yv(n,s2*r)/yv(n,s2*self.r2)


		def h_r_c2_clad(n,r,b):
			return s2*b*jv(n,s2*r)/(ss2*jv(n,s2*self.r2))

		def h_r_d2_clad(n,r,b):
			return s2*b*yv(n,s2*r)/(ss2*yv(n,s2*self.r2))
			
		def h_r_a2_clad(n,r,b):
			return self.o*ep*self.n_2*n*jv(n,s2*r)/(ss2*r*jv(n,s*self.r2))
			
		def h_r_b2_clad(n,r,b):
			return self.o*ep*self.n_2*n*yv(n,s2*r)/(ss2*r*yv(n,s*self.r2))
			

		def h_t_c2_clad(n,r,b):
			return b*n*jv(n,s2*r)/(ss2*r*jv(n,s2*self.r2))
			
		def h_t_d2_clad(n,r,b):
			return b*n*yv(n,s2*r)/(ss2*r*yv(n,s2*self.r2))
			
		def h_t_a2_clad(n,r,b):
			return s2*self.o*ep*self.n_2*jvp(n,s2*r)/(ss2*jv(n,s2*self.r2))

		def h_t_b2_clad(n,r,b):
			return s2*self.o*ep*self.n_2*yvp(n,s2*r)/(ss2*yv(n,s2*self.r2))
			

		def h_z_c2_clad(n,r,b):
			return jv(n,s2*r)/jv(n,s2*self.r2)

		def h_z_d2_clad(n,r,b):
			return yv(n,s2*r)/yv(n,s2*self.r2)


		# outer cladding

		def e_r_b3(n,r,b):
			return s3*b*kvp(n,s3*r)/(ss3*kv(n,s3*self.r2))

		def e_r_d3(n,r,b):
			return self.o*mu*n*kv(n,s3*r)/(ss3*r*kv(n,s3*self.r2))
				
		# e_theta
		def e_t_b3(n,r,b):
			return -b*n*kv(n,s3*r)/(ss3*r*kv(n,s3*self.r2))

		def e_t_d3(n,r,b):
			return -s3*self.o*mu*kvp(n,s3*r)/(ss3*kv(n,s3*self.r2))

		# e_z
		def e_z_b3(n,r,b):
			return kv(n,s3*r)/kv(n,s3*self.r2)

		# h_r	
		def h_r_d3(n,r,b):
			return s3*b*kvp(n,s3*r)/(ss3*kv(n,s3*self.r2))
			
		def h_r_b3(n,r,b):
			return self.o*ep*n*self.n_3*kv(n,s3*r)/(ss3*r*kv(n,s3*self.r2))

		# h_theta
		def h_t_d3(n,r,b):
			return b*n*kv(n,s3*r)/(ss3*r*kv(n,s3*self.r2))
			
		def h_t_b3(n,r,b):
			return s3*self.o*ep*self.n_3*kvp(n,s3*r)/(ss3*kv(n,s3*self.r2))
			
		#h_z
		def h_z_d3(n,r,b):
			return kv(n,s3*r)/kv(n,s3*self.r2)
			
		
		if self.n_2>ne:
			m01=e_z_a2_clad(self.m,self.r1,self.beta)
			m02=e_z_b2_clad(self.m,self.r1,self.beta)
			m11=e_z_a2_clad(self.m,self.r2,self.beta)
			m12=e_z_b2_clad(self.m,self.r2,self.beta)
			m25=h_z_c2_clad(self.m,self.r1,self.beta)
			m26=h_z_d2_clad(self.m,self.r1,self.beta)
			m35=h_z_c2_clad(self.m,self.r2,self.beta)
			m36=h_z_d2_clad(self.m,self.r2,self.beta)
			m41=e_t_a2_clad(self.m,self.r1,self.beta)
			m42=e_t_b2_clad(self.m,self.r1,self.beta)
			m45=e_t_c2_clad(self.m,self.r1,self.beta)
			m46=e_t_d2_clad(self.m,self.r1,self.beta)
			m51=e_t_a2_clad(self.m,self.r2,self.beta)
			m52=e_t_b2_clad(self.m,self.r2,self.beta)
			m55=e_t_c2_clad(self.m,self.r2,self.beta)
			m56=e_t_d2_clad(self.m,self.r2,self.beta)
			m61=h_t_a2_clad(self.m,self.r1,self.beta)
			m62=h_t_b2_clad(self.m,self.r1,self.beta)
			m65=h_t_c2_clad(self.m,self.r1,self.beta)
			m66=h_t_d2_clad(self.m,self.r1,self.beta)
			m71=h_t_a2_clad(self.m,self.r2,self.beta)
			m72=h_t_b2_clad(self.m,self.r2,self.beta)
			m75=h_t_c2_clad(self.m,self.r2,self.beta)
			m76=h_t_d2_clad(self.m,self.r2,self.beta)

		else:
			m01=e_z_a2_core(self.m,self.r1,self.beta)
			m02=e_z_b2_core(self.m,self.r1,self.beta)
			m11=e_z_a2_core(self.m,self.r2,self.beta)
			m12=e_z_b2_core(self.m,self.r2,self.beta)
			m25=h_z_c2_core(self.m,self.r1,self.beta)
			m26=h_z_d2_core(self.m,self.r1,self.beta)
			m35=h_z_c2_core(self.m,self.r2,self.beta)
			m36=h_z_d2_core(self.m,self.r2,self.beta)
			m41=e_t_a2_core(self.m,self.r1,self.beta)
			m42=e_t_b2_core(self.m,self.r1,self.beta)
			m45=e_t_c2_core(self.m,self.r1,self.beta)
			m46=e_t_d2_core(self.m,self.r1,self.beta)
			m51=e_t_a2_core(self.m,self.r2,self.beta)
			m52=e_t_b2_core(self.m,self.r2,self.beta)
			m55=e_t_c2_core(self.m,self.r2,self.beta)
			m56=e_t_d2_core(self.m,self.r2,self.beta)
			m61=h_t_a2_core(self.m,self.r1,self.beta)
			m62=h_t_b2_core(self.m,self.r1,self.beta)
			m65=h_t_c2_core(self.m,self.r1,self.beta)
			m66=h_t_d2_core(self.m,self.r1,self.beta)
			m71=h_t_a2_core(self.m,self.r2,self.beta)
			m72=h_t_b2_core(self.m,self.r2,self.beta)
			m75=h_t_c2_core(self.m,self.r2,self.beta)
			m76=h_t_d2_core(self.m,self.r2,self.beta)
		#self.r1 boundary
		m00=-e_z_a1(self.m,self.r1,self.beta)
		m03=0.
		m04=0.
		m05=0.
		m06=0.
		m07=0.
		#self.r2 boundary
		m10=0.

		m13=-e_z_b3(self.m,self.r2,self.beta)
		m14=0.
		m15=0.
		m16=0.
		m17=0.
		#h-field
		#self.r1 boundary
		m20=0.
		m21=0.
		m22=0.
		m23=0.
		m24=-h_z_c1(self.m,self.r1,self.beta)

		m27=0.
		#self.r2 boundary
		m30=0.
		m31=0.
		m32=0.
		m33=0.
		m34=0.

		m37=-h_z_d3(self.m,self.r2,self.beta)

		#theta equation
		#e-field
		#self.r1 boundary
		m40=-e_t_a1(self.m,self.r1,self.beta)

		m43=0.
		m44=-e_t_c1(self.m,self.r1,self.beta)

		m47=0.
		#self.r2 boundary
		m50=0.

		m53=-e_t_b3(self.m,self.r2,self.beta)
		m54=0.

		m57=-e_t_d3(self.m,self.r2,self.beta)
		#h-field
		#self.r1 boundary
		m60=-h_t_a1(self.m,self.r1,self.beta)

		m63=0.
		m64=-h_t_c1(self.m,self.r1,self.beta)

		m67=0.
		#self.r2 boundary
		m70=0.
		m73=-h_t_b3(self.m,self.r2,self.beta)
		m74=0.

		m77=-h_t_d3(self.m,self.r2,self.beta)

		m=[ [m00,m01,m02,m03,m04,m05,m06,m07],
			[m10,m11,m12,m13,m14,m15,m16,m17],
			[m20,m21,m22,m23,m24,m25,m26,m27],
			[m30,m31,m32,m33,m34,m35,m36,m37],
			[m40,m41,m42,m43,m44,m45,m46,m47],
			[m50,m51,m52,m53,m54,m55,m56,m57],
			[m60,m61,m62,m63,m64,m65,m66,m67],
			[m70,m71,m72,m73,m74,m75,m76,m77]]
		#~ print m
		return linalg.det(m)
		#~ return 1,([ [m00,m01,m02,m03,m04,m05,m06,m07],
			#~ [m10,m11,m12,m13,m14,m15,m16,m17],
			#~ [m20,m21,m22,m23,m24,m25,m26,m27],
			#~ [m30,m31,m32,m33,m34,m35,m36,m37],
			#~ [m40,m41,m42,m43,m44,m45,m46,m47],
			#~ [m50,m51,m52,m53,m54,m55,m56,m57],
			#~ [m60,m61,m62,m63,m64,m65,m66,m67],
			#~ [m70,m71,m72,m73,m74,m75,m76,m77]])
		#~ return mpmath.det(mpmath.matrix([[m00,m01,m02,m03,m04,m05,m06,m07],
								#~ [m10,m11,m12,m13,m14,m15,m16,m17],
								#~ [m20,m21,m22,m23,m24,m25,m26,m27],
								#~ [m30,m31,m32,m33,m34,m35,m36,m37],
								#~ [m40,m41,m42,m43,m44,m45,m46,m47],
								#~ [m50,m51,m52,m53,m54,m55,m56,m57],
								#~ [m60,m61,m62,m63,m64,m65,m66,m67],
								#~ [m70,m71,m72,m73,m74,m75,m76,m77]]))		
		
class main(wx.Frame):
	materials_name=[]
	materials_index=[]
	def __init__(self, *args, **kwds):
		
		# begin wxGlade: main.__init__
		wx.Frame.__init__(self, *args, **kwds)
		self.initialization()
		self.Label_wavelength = wx.StaticText(self, wx.ID_ANY, _("Start wavelength(nm)"))
		self.Text_wavelength = wx.TextCtrl(self, wx.ID_ANY, "500")
		self.Label_wavelength2 = wx.StaticText(self, wx.ID_ANY, _("Final wavelength (nm)"))
		self.Text_wavelength2 = wx.TextCtrl(self, wx.ID_ANY, "700")
		
		self.Label_r1 = wx.StaticText(self, wx.ID_ANY, _("Core diameter(um)"))
		self.Text_r1 = wx.TextCtrl(self, wx.ID_ANY, "8.2")
		self.Label_r2 = wx.StaticText(self, wx.ID_ANY, _("Clad. diameter (um)"))
		self.Text_r2 = wx.TextCtrl(self, wx.ID_ANY, "125")
		self.Label_core_material = wx.StaticText(self, wx.ID_ANY, _("Core material"))
		self.Box_core_material = wx.ComboBox(self, wx.ID_ANY, choices=self.materials_name, style=wx.CB_READONLY)
		self.Label_clad_material = wx.StaticText(self, wx.ID_ANY, _("Clad. material"))
		self.Box_clad_material = wx.ComboBox(self, wx.ID_ANY, choices=self.materials_name, style=wx.CB_READONLY)
		self.Label_air_material = wx.StaticText(self, wx.ID_ANY, _("Air material"))
		self.Box_air_material = wx.ComboBox(self, wx.ID_ANY, choices=self.materials_name, style=wx.CB_READONLY)
		self.Label_radial_mode = wx.StaticText(self, wx.ID_ANY, _("Radial mode number"))
		self.Text_radial_mode = wx.TextCtrl(self, wx.ID_ANY, "1")
		self.Label_step_size = wx.StaticText(self, wx.ID_ANY, _("Step size"))
		self.Text_step_size = wx.TextCtrl(self, wx.ID_ANY, "0.001")							
		self.Label_mode_wlstep = wx.StaticText(self,wx.ID_ANY, _("Wavelength steps"))
		self.Text_mode_wlstep = wx.TextCtrl(self, wx.ID_ANY, "1")
		self.Label_mode_limit = wx.StaticText(self,wx.ID_ANY, _("Max azimuthal mode"))
		self.Text_mode_limit = wx.TextCtrl(self, wx.ID_ANY, "4")
		self.Button_execute = wx.Button(self, wx.ID_ANY, _("Execute"))
		self.Button_graph = wx.Button(self, wx.ID_ANY, _("Graph"))
		self.Button_save_graph = wx.Button(self, wx.ID_ANY, _("Save graph"))
		self.Button_quit = wx.Button(self, wx.ID_ANY, _("Quit"))
		self.Box_core_material.Selection=1
		self.Box_clad_material.Selection=0
		self.Box_air_material.Selection=2
		self.Button_execute.Bind(wx.EVT_BUTTON, self.execute)
		self.Button_quit.Bind(wx.EVT_BUTTON, self.quitbtn)
		self.__set_properties()
		self.__do_layout()
		# end wxGlade

	def __set_properties(self):
		# begin wxGlade: main.__set_properties
		self.SetTitle(_("Radius"))
		# end wxGlade

	def __do_layout(self):
		# begin wxGlade: main.__do_layout
		sizer_1 = wx.BoxSizer(wx.VERTICAL)
		sizer_2 = wx.BoxSizer(wx.HORIZONTAL)
		grid_sizer_1 = wx.GridSizer(11, 2, 0, 0)
		grid_sizer_1.Add(self.Label_wavelength, 0, 0, 0)
		grid_sizer_1.Add(self.Text_wavelength, 0, 0, 0)
		grid_sizer_1.Add(self.Label_wavelength2, 0, 0, 0)
		grid_sizer_1.Add(self.Text_wavelength2, 0, 0, 0)
		grid_sizer_1.Add(self.Label_r1, 0, 0, 0)
		grid_sizer_1.Add(self.Text_r1, 0, 0, 0)
		grid_sizer_1.Add(self.Label_r2, 0, 0, 0)
		grid_sizer_1.Add(self.Text_r2, 0, 0, 0)
		grid_sizer_1.Add(self.Label_core_material, 0, 0, 0)
		grid_sizer_1.Add(self.Box_core_material, 0, 0, 0)
		grid_sizer_1.Add(self.Label_clad_material, 0, 0, 0)
		grid_sizer_1.Add(self.Box_clad_material, 0, 0, 0)
		grid_sizer_1.Add(self.Label_air_material, 0, 0, 0)
		grid_sizer_1.Add(self.Box_air_material, 0, 0, 0)
		grid_sizer_1.Add(self.Label_radial_mode, 0, 0, 0)
		grid_sizer_1.Add(self.Text_radial_mode, 0, 0, 0)
		grid_sizer_1.Add(self.Label_step_size, 0, 0, 0)
		grid_sizer_1.Add(self.Text_step_size, 0, 0, 0)
		grid_sizer_1.Add(self.Label_mode_limit,0,0,0)
		grid_sizer_1.Add(self.Text_mode_limit,0,0,0)
		grid_sizer_1.Add(self.Label_mode_wlstep,0,0,0)
		grid_sizer_1.Add(self.Text_mode_wlstep,0,0,0)
		sizer_1.Add(grid_sizer_1, 1, 0, 0)
		sizer_2.Add(self.Button_execute, 0, 0, 0)
		sizer_2.Add(self.Button_graph, 0, 0, 0)
		sizer_2.Add(self.Button_save_graph, 0, 0, 0)
		sizer_2.Add(self.Button_quit, 0, 0, 0)
		sizer_1.Add(sizer_2, 1, 0, 0)
		self.SetSizer(sizer_1)
		sizer_1.Fit(self)
		self.Layout()
		# end wxGlade
	def initialization(self):
		self.load_materials()

	def load_materials(self):
		d=0
		for f in os.listdir(os.getcwd()):
			if f.split(".")[-1]=="index":
				d=open(f,"r")
				self.materials_name+=[d.readline()[:-1]]
				tmps=d.readline()
				self.materials_index+=[tmps]
				d.close()
		return 0
	
	def execute(self, event):
		inittime=time.time()
		mm=multiprocessing.Manager()
		processes=[]
		finished=mm.list()
		radialmodes=mm.list()
		materials=mm.dict()
		
		materials['core']=self.materials_index[self.Box_core_material.GetCurrentSelection()]
		materials['clad']=self.materials_index[self.Box_clad_material.GetCurrentSelection()]
		materials['air']=self.materials_index[self.Box_air_material.GetCurrentSelection()]
		mode1=(self.Text_radial_mode.GetValue()).split(",")
		mode2=[]
		wl_start=float(self.Text_wavelength.GetValue())
		wl_final=float(self.Text_wavelength2.GetValue())
		wl_step=(wl_final-wl_start)/float(self.Text_mode_wlstep.GetValue())
		for i in mode1:
			t=i.split("-")
			if len(t)==1:
				mode2+=[int(i)]
			else:
				for j in range(int(t[0]),int(t[1])+1):
					mode2+=[j]
		print "In progress...\n"
		coreindex=open("core.txt","w")
		cladindex=open("clad.txt","w")
		airindex=open("air.txt","w")
		coreindexf=eval("lambda y:"+(self.materials_index[self.Box_core_material.GetCurrentSelection()]))
		cladindexf=eval("lambda y:"+(self.materials_index[self.Box_clad_material.GetCurrentSelection()]))
		airindexf=eval("lambda y:"+(self.materials_index[self.Box_air_material.GetCurrentSelection()]))
		for wltmp in np.arange(wl_start,wl_final,wl_step):
			coreindex.writelines(str(wltmp/1000.)+"\t"+str(coreindexf(wltmp/1000.))+"\n")
			cladindex.writelines(str(wltmp/1000.)+"\t"+str(cladindexf(wltmp/1000.))+"\n")
			airindex.writelines(str(wltmp/1000.)+"\t"+str(airindexf(wltmp/1000.))+"\n")
		for j in mode2:
			for wltmp in np.arange(wl_start,wl_final,wl_step):
				modeprocess=mode(radialmodes,finished,materials,
					r1=self.Text_r1.GetValue(),
					r2=self.Text_r2.GetValue(),
					wavelength=wltmp,
					mode_number=j,
					step=self.Text_step_size.GetValue(),
					limit=self.Text_mode_limit.GetValue()
					)
				modeprocess.daemon=True
				processes.append(modeprocess)
		processNo=len(processes)
		while len(processes):
			if processNo-len(finished)-len(processes)<psutil.cpu_count():
				processes.pop().start()
				print "processed units:", str(len(finished))+"/"+str(processNo),"still in progress..."
			else:
				time.sleep(0.1)
		while True:
			if processNo==len(finished):
				modelist={}
				gplfile=open("dataplot.gpl","w")
				gplfile.writelines("plot ")
				radialmodes.sort()
				print "Result writing..."
				for i in radialmodes:
					for k in i[1]:
						try:
							(modelist.get(k[0])).writelines(str(i[0])+"\t"+str(i[1])+"\t"+str(k[1])+"\n")
						except:
							modelist[k[0]]=open(k[0]+".txt","w")
							gplfile.writelines("\""+k[0]+".txt\" using 1:3 w l,")
							(modelist.get(k[0])).writelines(str(i[0])+"\t"+str(i[1])+"\t"+str(k[1])+"\n")
				print "Calculate Finished\n"
				print "Elapsed time: "+str(time.time()-inittime)+" sec."
				break
			else:
				print "processed units:", str(len(finished))+"/"+str(processNo),"still in progress..."
				time.sleep(1)
	def quitbtn(self,event):
		self.Close()
		
# end of class main

class myGraph(wx.MDIChildFrame):
	def __init__(self, *args, **kwds):
		# begin wxGlade: myGraph.__init__
		wx.MDIChildFrame.__init__(self, *args, **kwds)

		self.__set_properties()
		self.__do_layout()
		# end wxGlade

	def __set_properties(self):
		# begin wxGlade: myGraph.__set_properties
		self.SetTitle(_("frame_2"))
		# end wxGlade

	def __do_layout(self):
		# begin wxGlade: myGraph.__do_layout
		sizer_3 = wx.BoxSizer(wx.VERTICAL)
		self.SetSizer(sizer_3)
		sizer_3.Fit(self)
		self.Layout()
		# end wxGlade

# end of class myGraph
class MyApp(wx.App):
	def OnInit(self):
		mainframe = main(None, wx.ID_ANY, "")
		self.SetTopWindow(mainframe)
		mainframe.Show()
		return True

# end of class MyApp

if __name__ == "__main__":
	gettext.install("app") # replace with the appropriate catalog name
	app = MyApp(0)
	app.MainLoop()

