import multiprocessing
import numpy as np

#~ mpmath.seterr(all='raise')
np.seterr(all='ignore')

import numpy.linalg as linalg
import scipy.special as sp
import scipy.constants
from scipy.special import jv as jv
from scipy.special import yv as yv
from scipy.special import kv as kv
from scipy.special import iv as iv
from scipy.special import jvp as jvp
from scipy.special import yvp as yvp
from scipy.special import kvp as kvp
from scipy.special import ivp as ivp
from scipy.special import jve as jve
from scipy.special import yve as yve
from scipy.special import kve as kve
from scipy.special import ive as ive


from scipy.optimize import brentq as rootfind
#~ from mpmath import findroot as rootfind
mu=scipy.constants.mu_0
ep=scipy.constants.epsilon_0
speed=scipy.constants.speed_of_light


def modeIdentify(p,q):
	if p==0:
		if q%2==1:
			return "TM0"+str((q+1)/2)
		else:
			return "TE0"+str(q/2)
	else:
		if q%2==1:
			return "HE"+str(p)+str((q+1)/2)
		else:
			return "EH"+str(p)+str(q/2)
			
	return "No such mode!\n"


class mode(multiprocessing.Process):
	modes=[]
	def __init__(self,radialmodes,finished,materials, **kwds):
		multiprocessing.Process.__init__(self)
		self.finished=finished
		self.modes=[]
		self.radialmodes=radialmodes
		self.r1=float(kwds.pop('r1'))*scipy.constants.micro/2.
		self.r2=float(kwds.pop('r2'))*scipy.constants.micro/2.
		self.wl=float(kwds.pop('wavelength'))/1000.
		self.m=int(kwds.pop('mode_number'))
		self.step=1./float(kwds.pop('step'))
		self.n1=materials['core']
		self.n2=materials['clad']
		self.n3=materials['air']
		self.limit=int(kwds.pop('limit'))
		self.k = np.pi*2000000./self.wl # wavenumber
		self.o = speed*self.k # angular frequency of the light.
		self.oep=self.o*ep
		self.omu=self.o*mu
	def run(self):
		try:
			
			self.n1=eval("lambda y:"+(self.n1))
			self.n2=eval("lambda y:"+(self.n2))
			self.n3=eval("lambda y:"+(self.n3))

			self.n_1,self.n_2,self.n_3=self.n1(self.wl),self.n2(self.wl),self.n3(self.wl)
			self.n_1*=self.n_1
			self.n_2*=self.n_2
			self.n_3*=self.n_3
			indexStep=(self.n1(self.wl)-self.n2(self.wl))/self.step
			j=(self.n1(self.wl))-indexStep
			
			myVal0=self.layerdeterminant(j)
			myVal=myVal0
			v=1
			#~ targetfile=open("mp"+str(os.getpid())+".txt","a")
			for i in np.arange(self.n1(self.wl)-indexStep,self.n2(self.wl)+indexStep,-indexStep):
				myVal0=myVal
				myVal=self.layerdeterminant((i))
				#~ print type(myVal),myVal , i
				#~ targetfile.writelines(str(self.wl)+","+str(i)+","+str(myVal[1])+"\n")
				if myVal*myVal0<0.:	
					self.modes+=[[modeIdentify(self.m,v),rootfind(self.layerdeterminant,(i),(j))]]
					v+=1
				if v>self.limit:
					break
				j=i
			indexStep=(self.n2(self.wl)-self.n3(self.wl))/self.step
			j=(self.n3(self.wl))
			myVal0=self.layerdeterminant((i))
			myVal=myVal0
			for i in np.arange(self.n2(self.wl)-indexStep,self.n3(self.wl)+indexStep,-indexStep):
				myVal0=myVal
				myVal=self.layerdeterminant((i))
				if myVal*myVal0<0.:	
					self.modes+=[[modeIdentify(self.m,v),rootfind(self.layerdeterminant,(i),(j))]]
					v+=1
				if v>self.limit:
					break
				j=i
		finally:
			self.radialmodes.append([self.wl,self.r2/scipy.constants.micro,self.modes])
			self.finished.append(1)
		
	def showparameters(self):
		print "r1",self.r1
		print "r2",self.r2
		print "wl",self.wl
		print "mode",self.m
		print "n1",self.n1(self.wl)
		print "n2",self.n2(self.wl)
		print "n3",self.n3(self.wl)
			
	def layerdeterminant(self,nef):
		# boundary condition
		self.beta=self.k*nef
		ne=nef*nef
		s1=self.k*np.sqrt(self.n_1-ne) # core
		if self.n_2>ne:
			s2=self.k*np.sqrt(self.n_2-ne) # clad mode
		else:
			s2=self.k*np.sqrt(ne-self.n_2) # core mode
		s3=self.k*np.sqrt(ne-self.n_3) # clad
		
		ss1=self.k*self.k*(self.n_1-ne)
		ss2=self.k*self.k*(self.n_2-ne)
		ss3=self.k*self.k*(self.n_3-ne)
		def e_r_a1(n,r,b):
			return s1*b*jvp(n,s1*r)/(ss1*jv(n,s1*self.r1))

		def e_r_c1(n,r,b):
			return self.o*mu*n*jv(n,s1*r)/(ss1*r*jv(n,s1*self.r1))
				
		# e_theta
		def e_t_a1(n,r,b):
			return -b*n*jv(n,s1*r)/(ss1*r*jv(n,s1*self.r1))

		def e_t_c1(n,r,b):
			return -s1*self.o*mu*jvp(n,s1*r)/(ss1*jv(n,s1*self.r1))

		# e_z
		def e_z_a1(n,r,b):
			return jv(n,s1*r)/jv(n,s1*self.r1)

		# h_r	
		def h_r_c1(n,r,b):
			return s1*b*jvp(n,s1*r)/(ss1*jv(n,s1*self.r1))
			
		def h_r_a1(n,r,b):
			return self.o*ep*n*self.n_1*jv(n,s1*r)/(ss1*r*jv(n,s1*self.r1))

		# h_theta
		def h_t_c1(n,r,b):
			return b*n*jv(n,s1*r)/(ss1*r*jv(n,s1*self.r1))
			
		def h_t_a1(n,r,b):
			return s1*self.o*ep*self.n_1*jvp(n,s1*r)/(ss1*jv(n,s1*self.r1))
			
		#h_z
		def h_z_c1(n,r,b):
			return jv(n,s1*r)/jv(n,s1*self.r1)

		# inner cladding core mode
		# if clad mode: iv, kv, self.r2 -> jv, yv, self.r1, repectively.

		def e_r_a2_core(n,r,b):
			return s2*b*ivp(n,s2*r)/(ss2*iv(n,s2*self.r2))

		def e_r_b2_core(n,r,b):
			return s2*b*kvp(n,s2*r)/(ss2*kv(n,s2*self.r2))

		def e_r_c2_core(n,r,b):
			return self.o*mu*n*iv(n,s2*r)/(ss2*r*iv(n,s2*self.r2))

		def e_r_d2_core(n,r,b):
			return self.o*mu*n*kv(n,s2*r)/(ss2*r*kv(n,s2*self.r2))


		def e_t_a2_core(n,r,b):
			return -b*n*iv(n,s2*r)/(ss2*r*iv(n,s2*self.r2))
			
		def e_t_b2_core(n,r,b):
			return -b*n*kv(n,s2*r)/(ss2*r*kv(n,s2*self.r2))

		def e_t_c2_core(n,r,b):
			return -s2*self.o*mu*ivp(n,s2*r)/(ss2*iv(n,s2*self.r2))
			
		def e_t_d2_core(n,r,b):
			return -s2*self.o*mu*kvp(n,s2*r)/(ss2*kv(n,s2*self.r2))
			

		def e_z_a2_core(n,r,b):
			#~ print type(s2),type(r),type(self.r2)
			return iv(n,s2*r)/iv(n,s2*self.r2)
			
		def e_z_b2_core(n,r,b):
			return kv(n,s2*r)/kv(n,s2*self.r2)


		def h_r_c2_core(n,r,b):
			return s2*b*iv(n,s2*r)/(ss2*iv(n,s2*self.r2))

		def h_r_d2_core(n,r,b):
			return s2*b*kv(n,s2*r)/(ss2*kv(n,s2*self.r2))
			
		def h_r_a2_core(n,r,b):
			return self.o*ep*self.n_2*n*iv(n,s2*r)/(ss2*r*iv(n,s*self.r2))
			
		def h_r_b2_core(n,r,b):
			return self.o*ep*self.n_2*n*kv(n,s2*r)/(ss2*r*kv(n,s*self.r2))
			

		def h_t_c2_core(n,r,b):
			return b*n*iv(n,s2*r)/(ss2*r*iv(n,s2*self.r2))
			
		def h_t_d2_core(n,r,b):
			return b*n*kv(n,s2*r)/(ss2*r*kv(n,s2*self.r2))
			
		def h_t_a2_core(n,r,b):
			return s2*self.o*ep*self.n_2*ivp(n,s2*r)/(ss2*iv(n,s2*self.r2))

		def h_t_b2_core(n,r,b):
			return s2*self.o*ep*self.n_2*kvp(n,s2*r)/(ss2*kv(n,s2*self.r2))
			

		def h_z_c2_core(n,r,b):
			#~ print iv(n,s2*r)/iv(n,s2*self.r2)
			return iv(n,s2*r)/iv(n,s2*self.r2)

		def h_z_d2_core(n,r,b):
			return kv(n,s2*r)/kv(n,s2*self.r2)

		# inner cladding core mode
		# if clad mode: iv, kv, self.r2 -> jv, yv, self.r1, repectively.

		def e_r_a2_clad(n,r,b):
			return s2*b*jvp(n,s2*r)/(ss2*jv(n,s2*self.r2))

		def e_r_b2_clad(n,r,b):
			return s2*b*yvp(n,s2*r)/(ss2*yv(n,s2*self.r2))

		def e_r_c2_clad(n,r,b):
			return self.o*mu*n*jv(n,s2*r)/(ss2*r*jv(n,s2*self.r2))

		def e_r_d2_clad(n,r,b):
			return self.o*mu*n*yv(n,s2*r)/(ss2*r*yv(n,s2*self.r2))


		def e_t_a2_clad(n,r,b):
			return -b*n*jv(n,s2*r)/(ss2*r*jv(n,s2*self.r2))
			
		def e_t_b2_clad(n,r,b):
			return -b*n*yv(n,s2*r)/(ss2*r*yv(n,s2*self.r2))

		def e_t_c2_clad(n,r,b):
			return -s2*self.o*mu*jvp(n,s2*r)/(ss2*jv(n,s2*self.r2))
			
		def e_t_d2_clad(n,r,b):
			return -s2*self.o*mu*yvp(n,s2*r)/(ss2*yv(n,s2*self.r2))
			

		def e_z_a2_clad(n,r,b):
			return jv(n,s2*r)/jv(n,s2*self.r2)
			
		def e_z_b2_clad(n,r,b):
			return yv(n,s2*r)/yv(n,s2*self.r2)


		def h_r_c2_clad(n,r,b):
			return s2*b*jv(n,s2*r)/(ss2*jv(n,s2*self.r2))

		def h_r_d2_clad(n,r,b):
			return s2*b*yv(n,s2*r)/(ss2*yv(n,s2*self.r2))
			
		def h_r_a2_clad(n,r,b):
			return self.o*ep*self.n_2*n*jv(n,s2*r)/(ss2*r*jv(n,s*self.r2))
			
		def h_r_b2_clad(n,r,b):
			return self.o*ep*self.n_2*n*yv(n,s2*r)/(ss2*r*yv(n,s*self.r2))
			

		def h_t_c2_clad(n,r,b):
			return b*n*jv(n,s2*r)/(ss2*r*jv(n,s2*self.r2))
			
		def h_t_d2_clad(n,r,b):
			return b*n*yv(n,s2*r)/(ss2*r*yv(n,s2*self.r2))
			
		def h_t_a2_clad(n,r,b):
			return s2*self.o*ep*self.n_2*jvp(n,s2*r)/(ss2*jv(n,s2*self.r2))

		def h_t_b2_clad(n,r,b):
			return s2*self.o*ep*self.n_2*yvp(n,s2*r)/(ss2*yv(n,s2*self.r2))
			

		def h_z_c2_clad(n,r,b):
			return jv(n,s2*r)/jv(n,s2*self.r2)

		def h_z_d2_clad(n,r,b):
			return yv(n,s2*r)/yv(n,s2*self.r2)


		# outer cladding

		def e_r_b3(n,r,b):
			return s3*b*kvp(n,s3*r)/(ss3*kv(n,s3*self.r2))

		def e_r_d3(n,r,b):
			return self.o*mu*n*kv(n,s3*r)/(ss3*r*kv(n,s3*self.r2))
				
		# e_theta
		def e_t_b3(n,r,b):
			return -b*n*kv(n,s3*r)/(ss3*r*kv(n,s3*self.r2))

		def e_t_d3(n,r,b):
			return -s3*self.o*mu*kvp(n,s3*r)/(ss3*kv(n,s3*self.r2))

		# e_z
		def e_z_b3(n,r,b):
			return kv(n,s3*r)/kv(n,s3*self.r2)

		# h_r	
		def h_r_d3(n,r,b):
			return s3*b*kvp(n,s3*r)/(ss3*kv(n,s3*self.r2))
			
		def h_r_b3(n,r,b):
			return self.o*ep*n*self.n_3*kv(n,s3*r)/(ss3*r*kv(n,s3*self.r2))

		# h_theta
		def h_t_d3(n,r,b):
			return b*n*kv(n,s3*r)/(ss3*r*kv(n,s3*self.r2))
			
		def h_t_b3(n,r,b):
			return s3*self.o*ep*self.n_3*kvp(n,s3*r)/(ss3*kv(n,s3*self.r2))
			
		#h_z
		def h_z_d3(n,r,b):
			return kv(n,s3*r)/kv(n,s3*self.r2)
			
		
		if self.n_2>ne:
			m01=e_z_a2_clad(self.m,self.r1,self.beta)
			m02=e_z_b2_clad(self.m,self.r1,self.beta)
			m11=e_z_a2_clad(self.m,self.r2,self.beta)
			m12=e_z_b2_clad(self.m,self.r2,self.beta)
			m25=h_z_c2_clad(self.m,self.r1,self.beta)
			m26=h_z_d2_clad(self.m,self.r1,self.beta)
			m35=h_z_c2_clad(self.m,self.r2,self.beta)
			m36=h_z_d2_clad(self.m,self.r2,self.beta)
			m41=e_t_a2_clad(self.m,self.r1,self.beta)
			m42=e_t_b2_clad(self.m,self.r1,self.beta)
			m45=e_t_c2_clad(self.m,self.r1,self.beta)
			m46=e_t_d2_clad(self.m,self.r1,self.beta)
			m51=e_t_a2_clad(self.m,self.r2,self.beta)
			m52=e_t_b2_clad(self.m,self.r2,self.beta)
			m55=e_t_c2_clad(self.m,self.r2,self.beta)
			m56=e_t_d2_clad(self.m,self.r2,self.beta)
			m61=h_t_a2_clad(self.m,self.r1,self.beta)
			m62=h_t_b2_clad(self.m,self.r1,self.beta)
			m65=h_t_c2_clad(self.m,self.r1,self.beta)
			m66=h_t_d2_clad(self.m,self.r1,self.beta)
			m71=h_t_a2_clad(self.m,self.r2,self.beta)
			m72=h_t_b2_clad(self.m,self.r2,self.beta)
			m75=h_t_c2_clad(self.m,self.r2,self.beta)
			m76=h_t_d2_clad(self.m,self.r2,self.beta)

		else:
			m01=e_z_a2_core(self.m,self.r1,self.beta)
			m02=e_z_b2_core(self.m,self.r1,self.beta)
			m11=e_z_a2_core(self.m,self.r2,self.beta)
			m12=e_z_b2_core(self.m,self.r2,self.beta)
			m25=h_z_c2_core(self.m,self.r1,self.beta)
			m26=h_z_d2_core(self.m,self.r1,self.beta)
			m35=h_z_c2_core(self.m,self.r2,self.beta)
			m36=h_z_d2_core(self.m,self.r2,self.beta)
			m41=e_t_a2_core(self.m,self.r1,self.beta)
			m42=e_t_b2_core(self.m,self.r1,self.beta)
			m45=e_t_c2_core(self.m,self.r1,self.beta)
			m46=e_t_d2_core(self.m,self.r1,self.beta)
			m51=e_t_a2_core(self.m,self.r2,self.beta)
			m52=e_t_b2_core(self.m,self.r2,self.beta)
			m55=e_t_c2_core(self.m,self.r2,self.beta)
			m56=e_t_d2_core(self.m,self.r2,self.beta)
			m61=h_t_a2_core(self.m,self.r1,self.beta)
			m62=h_t_b2_core(self.m,self.r1,self.beta)
			m65=h_t_c2_core(self.m,self.r1,self.beta)
			m66=h_t_d2_core(self.m,self.r1,self.beta)
			m71=h_t_a2_core(self.m,self.r2,self.beta)
			m72=h_t_b2_core(self.m,self.r2,self.beta)
			m75=h_t_c2_core(self.m,self.r2,self.beta)
			m76=h_t_d2_core(self.m,self.r2,self.beta)
		#self.r1 boundary
		m00=-e_z_a1(self.m,self.r1,self.beta)
		m03=0.
		m04=0.
		m05=0.
		m06=0.
		m07=0.
		#self.r2 boundary
		m10=0.

		m13=-e_z_b3(self.m,self.r2,self.beta)
		m14=0.
		m15=0.
		m16=0.
		m17=0.
		#h-field
		#self.r1 boundary
		m20=0.
		m21=0.
		m22=0.
		m23=0.
		m24=-h_z_c1(self.m,self.r1,self.beta)

		m27=0.
		#self.r2 boundary
		m30=0.
		m31=0.
		m32=0.
		m33=0.
		m34=0.

		m37=-h_z_d3(self.m,self.r2,self.beta)

		#theta equation
		#e-field
		#self.r1 boundary
		m40=-e_t_a1(self.m,self.r1,self.beta)

		m43=0.
		m44=-e_t_c1(self.m,self.r1,self.beta)

		m47=0.
		#self.r2 boundary
		m50=0.

		m53=-e_t_b3(self.m,self.r2,self.beta)
		m54=0.

		m57=-e_t_d3(self.m,self.r2,self.beta)
		#h-field
		#self.r1 boundary
		m60=-h_t_a1(self.m,self.r1,self.beta)

		m63=0.
		m64=-h_t_c1(self.m,self.r1,self.beta)

		m67=0.
		#self.r2 boundary
		m70=0.
		m73=-h_t_b3(self.m,self.r2,self.beta)
		m74=0.

		m77=-h_t_d3(self.m,self.r2,self.beta)

		m=[ [m00,m01,m02,m03,m04,m05,m06,m07],
			[m10,m11,m12,m13,m14,m15,m16,m17],
			[m20,m21,m22,m23,m24,m25,m26,m27],
			[m30,m31,m32,m33,m34,m35,m36,m37],
			[m40,m41,m42,m43,m44,m45,m46,m47],
			[m50,m51,m52,m53,m54,m55,m56,m57],
			[m60,m61,m62,m63,m64,m65,m66,m67],
			[m70,m71,m72,m73,m74,m75,m76,m77]]
		#~ print m
		return linalg.det(m)
		#~ return 1,([ [m00,m01,m02,m03,m04,m05,m06,m07],
			#~ [m10,m11,m12,m13,m14,m15,m16,m17],
			#~ [m20,m21,m22,m23,m24,m25,m26,m27],
			#~ [m30,m31,m32,m33,m34,m35,m36,m37],
			#~ [m40,m41,m42,m43,m44,m45,m46,m47],
			#~ [m50,m51,m52,m53,m54,m55,m56,m57],
			#~ [m60,m61,m62,m63,m64,m65,m66,m67],
			#~ [m70,m71,m72,m73,m74,m75,m76,m77]])
		#~ return mpmath.det(mpmath.matrix([[m00,m01,m02,m03,m04,m05,m06,m07],
								#~ [m10,m11,m12,m13,m14,m15,m16,m17],
								#~ [m20,m21,m22,m23,m24,m25,m26,m27],
								#~ [m30,m31,m32,m33,m34,m35,m36,m37],
								#~ [m40,m41,m42,m43,m44,m45,m46,m47],
								#~ [m50,m51,m52,m53,m54,m55,m56,m57],
								#~ [m60,m61,m62,m63,m64,m65,m66,m67],
								#~ [m70,m71,m72,m73,m74,m75,m76,m77]]))		
